# -*- coding: utf-8 -*-
from django.dispatch import Signal

setup_test_environment = Signal()
teardown_test_environment = Signal()

before_suite_run = Signal()
after_suite_run = Signal()

build_suite = Signal(providing_args=["suite"])



# -*- coding: utf-8 -*-
import os
import sys
import socket
import tempfile
import threading
import unittest
from imp import find_module
from optparse import make_option
from unittest import _strclass
from windmill.bin import admin_lib
from windmill.authoring import WindmillTestClient
from django.db.models import get_app, get_apps
from django.conf import settings
from django.core import urlresolvers
from django.core.handlers.wsgi import WSGIHandler
from django.core.servers import basehttp
from django.test import TestCase, TransactionTestCase
from django_jenkins.tasks import BaseTask


if sys.version_info[1] < 5:
    raise AssertionError('windmill task require python>=2.5 version')

import windmill

WM_TEST_MODULE = 'wmtests'
TEST_SERVER_HOST = '127.0.0.2'
TEST_SERVER_PORT = 0


def get_tests(app_module):
    try:
        app_path = app_module.__name__.split('.')[:-1]
        test_module = __import__('.'.join(app_path + [WM_TEST_MODULE]), {}, {}, WM_TEST_MODULE)
    except ImportError:
        # Couldn't import tests.py. Was it due to a missing file, or
        # due to an import error in a tests.py that actually exists?
        try:
            mod = find_module(WM_TEST_MODULE, [os.path.dirname(app_module.__file__)])
        except ImportError:
            # 'tests' module doesn't exist. Move on.
            test_module = None
        else:
            # The module exists, so there must be an import error in the
            # test module itself. We don't need the module; so if the
            # module was a single file module (i.e., tests.py), close the file
            # handle returned by find_module. Otherwise, the test module
            # is a directory, and there is nothing to close.
            if mod[0]:
                mod[0].close()
            raise
    return test_module


def build_suite(app_module):
    """
    Create a complete Django test suite for the provided application module
    """
    suite = unittest.TestSuite()

    # Load unit and doctests in the models.py module. If module has
    # a suite() method, use it. Otherwise build the test suite ourselves.
    if hasattr(app_module, 'suite'):
        suite.addTest(app_module.suite())
    else:
        suite.addTest(unittest.defaultTestLoader.loadTestsFromModule(app_module))

    # Check to see if a separate 'tests' module exists parallel to the
    # models module
    test_module = get_tests(app_module)
    if test_module:
        # Load unit and doctests in the tests.py module. If module has
        # a suite() method, use it. Otherwise build the test suite ourselves.
        if hasattr(test_module, 'suite'):
            suite.addTest(test_module.suite())
        else:
            suite.addTest(unittest.defaultTestLoader.loadTestsFromModule(test_module))

    return suite


def build_test(label):
    """
    Construct a test case with the specified label. Label should be of the
    form model.TestClass or model.TestClass.test_method. Returns an
    instantiated test or test suite corresponding to the label provided.

    """
    parts = label.split('.')
    if len(parts) < 2 or len(parts) > 3:
        raise ValueError("Test label '%s' should be of the form app.TestCase or app.TestCase.test_method" % label)

    #
    # First, look for TestCase instances with a name that matches
    #
    app_module = get_app(parts[0])
    test_module = get_tests(app_module)
    TestClass = getattr(app_module, parts[1], None)

    # Couldn't find the test class in models.py; look in tests.py
    if TestClass is None:
        if test_module:
            TestClass = getattr(test_module, parts[1], None)

    try:
        if issubclass(TestClass, unittest.TestCase):
            if len(parts) == 2:  # label is app.TestClass
                try:
                    return unittest.TestLoader().loadTestsFromTestCase(TestClass)
                except TypeError:
                    raise ValueError("Test label '%s' does not refer to a test class" % label)
            else:  # label is app.TestClass.test_method
                return TestClass(parts[2])
    except TypeError:
        # TestClass isn't a TestClass - it must be a method or normal class
        pass


class StoppableWSGIServer(basehttp.WSGIServer):
    """WSGIServer with short timeout, so that server thread can stop this server."""

    def server_bind(self):
        """Sets timeout to 1 second."""
        basehttp.WSGIServer.server_bind(self)
        self.socket.settimeout(1)

        global TEST_SERVER_HOST, TEST_SERVER_PORT
        TEST_SERVER_HOST, TEST_SERVER_PORT = self.socket.getsockname()
        if TEST_SERVER_PORT == '0.0.0.0':
            TEST_SERVER_HOST = '127.0.0.2'

    def get_request(self):
        """Checks for timeout when getting request."""
        try:
            sock, address = self.socket.accept()
            sock.settimeout(None)
            return (sock, address)
        except socket.timeout:
            raise


class TestServerThread(threading.Thread):
    """Thread for running a http server while tests are running."""

    def __init__(self, server_addr):
        self.server_addr = server_addr
        self._stopevent = threading.Event()
        self.started = threading.Event()
        self.error = None
        super(TestServerThread, self).__init__()

    def run(self):
        """Sets up test server and loops over handling http requests."""
        try:
            handler = basehttp.AdminMediaHandler(WSGIHandler())
            httpd = StoppableWSGIServer(self.server_addr, basehttp.WSGIRequestHandler)
            httpd.application = handler
            self.started.set()
        except basehttp.WSGIServerException, err:
            self.error = err
            self.started.set()
            return

        # Loop until we get a stop event.
        while not self._stopevent.isSet():
            httpd.handle_request()

    def join(self, timeout=None):
        """Stop the thread and wait for it to finish."""
        self._stopevent.set()
        threading.Thread.join(self, timeout)


class Task(BaseTask):
    option_list = [make_option("--with-browser",
                               dest="browsers", action="append", default=None,
                               help="Select browser for test. This option could be added multiple times")]

    def __init__(self, test_labels, options):
        super(Task, self).__init__(test_labels, options)
        self.windmill_cmds = {}
        self.browsers = options['browsers'] or ["firefox"]
        self.output_dir = options.get('output_dir', 'reports')
        self.verbosity = int(options.get('verbosity', 1))
        self.test_server_host = getattr(settings, 'WINDMILL_HOST', '127.0.0.2')  # for 127.0.0.1 FF always ignore proxy for me
        self.test_server_port = getattr(settings, 'WINDMILL_PORT', 0)  # select random available port
        if not self.test_labels:
            if hasattr(settings, 'PROJECT_APPS') and not options['test_all']:
                self.test_labels = [app_name.split('.')[-1] for app_name in settings.PROJECT_APPS]

    def setup_test_environment(self, **kwargs):
        #configure windmill
        admin_lib.configure_global_settings(logging_on=False)
        windmill.settings['shell_objects'] = self.windmill_cmds
        
        self.windmill_cmds['httpd'], self.windmill_cmds['httpd_thread'] = \
            admin_lib.run_threaded(windmill.settings['CONSOLE_LOG_LEVEL'])

        from windmill.bin import shell_objects
        for attribute in dir(shell_objects):
            self.windmill_cmds[attribute] = getattr(shell_objects, attribute)

        self.windmill_cmds['setup_has_run'] = True

        # firefox 4.0 compatibiliy hack
        if 'MOZILLA_DEFAULT_PROFILE' not in windmill.settings:
            tmp_profile = tempfile.mkdtemp(suffix='.mozrunner')
            windmill.settings['MOZILLA_DEFAULT_PROFILE']= tmp_profile

        #run wsgi django server
        self.server_thread = TestServerThread((self.test_server_host, self.test_server_port))
        self.server_thread.start()
        self.server_thread.started.wait()
        if self.server_thread.error:
            raise self.server_thread.error

    def teardown_test_environment(self, **kwargs):
        # stop windmill server
        self.windmill_cmds['httpd'].stop()

        # stop django wsgi server
        if self.server_thread:
            self.server_thread.join()

    def build_suite(self, suite, **kwargs):
        for browser in self.browsers:
            windmill_suite = WindmillTestSuite(windmill_cmds=self.windmill_cmds, browser=browser)

            if self.test_labels:
                for label in self.test_labels:
                    if '.' in label:
                        windmill_suite.addTest(build_test(label))
                    else:
                        app = get_app(label)
                        windmill_suite.addTest(build_suite(app))
            else:
                for app in get_apps():
                    windmill_suite.addTest(build_suite(app))

            suite.addTest(windmill_suite)

        return suite


class WindmillTestSuite(unittest.TestSuite):
    def __init__(self, windmill_cmds, browser, **kwargs):
        super(WindmillTestSuite, self).__init__(**kwargs)
        self.windmill_cmds = windmill_cmds
        self.browser = browser

    def __call__(self, *args, **kwargs):
        try:
            # Need to enshure, that standard streams have original values
            # elsewhere Popen windmill calls fails
            old_stdout, old_stdin, old_stderr = sys.stdout, sys.stdin, sys.stderr
            sys.stdout, sys.stdin, sys.stderr = sys.__stdout__, sys.__stdin__, sys.__stderr__

            # configure windmill
            windmill.settings['active_browser'] = self.browser
            self.windmill_cmds['start_' + self.browser]()

            # restore
            sys.stdout, sys.stdin, sys.stderr = old_stdout, old_stdin, old_stderr

            # run tests
            super(WindmillTestSuite, self).__call__(*args, **kwargs)
        finally:
            # stop windmill browser
            self.windmill_cmds['clear_queue']()
            for controller in windmill.settings['controllers']:
                controller.stop()
            windmill.settings['controllers'] = []


class WindmillMixin(object):
    @property
    def base_url(self):
        return "http://%s:%s" % (TEST_SERVER_HOST, TEST_SERVER_PORT)

    def shortDescription(self):
            return "%s (%s) under browser %s" % (self._testMethodName,  _strclass(self.__class__), windmill.settings['active_browser'])

    def __call__(self, result=None):
        self.windmill = WindmillTestClient(__name__)
        old_opener = self.windmill.open

        def opener(url, *args, **kwargs):
            if url.startswith('http'):
                return old_opener(url=url)

            if not url.startswith('/'):
                path = urlresolvers.reverse(url, args=args, kwargs=kwargs)
            else:
                path = url

            return old_opener(url=self.base_url + path)
        self.windmill.open = opener

        return super(TestCase, self).__call__(result=result)


class WindmillTransactionalTestCase(WindmillMixin, TransactionTestCase):
    pass


class WindmillTestCase(WindmillMixin, TestCase):
    pass



# -*- coding: utf-8 -*-
"""
Extensible dango test runner
"""
import os
import sys
import time
import unittest
from cStringIO import StringIO
from unittest import _TextTestResult, TestResult
from xml.dom.minidom import Document
from django.conf import settings
from django.test.simple import DjangoTestSuiteRunner, DjangoTestRunner
from django_jenkins import signals


class _TestInfo(object):
    """This class is used to keep useful information about the execution of a
    test method.
    """

    # Possible test outcomes
    (SUCCESS, FAILURE, ERROR) = range(3)

    def __init__(self, test_result, test_method, outcome=SUCCESS, err=None):
        "Create a new instance of _TestInfo."
        self.test_result = test_result
        self.test_method = test_method
        self.outcome = outcome
        self.err = err

    def get_elapsed_time(self):
        """Return the time that shows how long the test method took to
        execute.
        """
        return self.test_result.stop_time - self.test_result.start_time

    def get_description(self):
        "Return a text representation of the test method."
        return self.test_result.getDescription(self.test_method)

    def get_error_info(self):
        """Return a text representation of an exception thrown by a test
        method.
        """
        if not self.err:
            return ''

        return self.test_result._exc_info_to_string(self.err, \
            self.test_method)


class _XMLTestResult(_TextTestResult):
    """
    A test result class that can express test results in a XML report.

    Used by XMLTestRunner.
    """
    def __init__(self, stream=sys.stderr, descriptions=1, verbosity=1, \
                 elapsed_times=True):
        "Create a new instance of _XMLTestResult."
        _TextTestResult.__init__(self, stream, descriptions, verbosity)
        self.successes = []
        self.callback = None
        self.elapsed_times = elapsed_times

    def _prepare_callback(self, test_info, target_list, verbose_str, \
                          short_str):
        """Append a _TestInfo to the given target list and sets a callback
        method to be called by stopTest method.
        """
        target_list.append(test_info)

        def callback():
            """This callback prints the test method outcome to the stream,
            as well as the elapsed time.
            """

            # Ignore the elapsed times for a more reliable unit testing
            if not self.elapsed_times:
                self.start_time = self.stop_time = 0

            if self.showAll:
                self.stream.writeln('%s (%.3fs)' % \
                    (verbose_str, test_info.get_elapsed_time()))
            elif self.dots:
                self.stream.write(short_str)
        self.callback = callback

    def startTest(self, test):
        "Called before execute each test method."
        self.start_time = time.time()
        TestResult.startTest(self, test)

        if self.showAll:
            self.stream.write('  ' + self.getDescription(test))
            self.stream.write(" ... ")

    def stopTest(self, test):
        "Called after execute each test method."
        _TextTestResult.stopTest(self, test)
        self.stop_time = time.time()

        if self.callback and callable(self.callback):
            self.callback()
            self.callback = None

    def addSuccess(self, test):
        "Called when a test executes successfully."
        self._prepare_callback(_TestInfo(self, test), \
            self.successes, 'OK', '.')

    def addFailure(self, test, err):
        "Called when a test method fails."
        self._prepare_callback(_TestInfo(self, test, _TestInfo.FAILURE, err), \
            self.failures, 'FAIL', 'F')

    def addError(self, test, err):
        "Called when a test method raises an error."
        self._prepare_callback(_TestInfo(self, test, _TestInfo.ERROR, err), \
            self.errors, 'ERROR', 'E')

    def printErrorList(self, flavour, errors):
        "Write some information about the FAIL or ERROR to the stream."
        for test_info in errors:
            self.stream.writeln(self.separator1)
            self.stream.writeln('%s [%.3fs]: %s' % \
                (flavour, test_info.get_elapsed_time(), \
                test_info.get_description()))
            self.stream.writeln(self.separator2)
            self.stream.writeln('%s' % test_info.get_error_info())

    def _get_info_by_testcase(self):
        """This method organizes test results by TestCase module. This
        information is used during the report generation, where a XML report
        will be generated for each TestCase.
        """
        tests_by_testcase = {}

        for tests in (self.successes, self.failures, self.errors):
            for test_info in tests:
                testcase = type(test_info.test_method)

                # Ignore module name if it is '__main__'
                module = testcase.__module__ + '.'
                if module == '__main__.':
                    module = ''
                testcase_name = module + testcase.__name__

                if testcase_name not in tests_by_testcase:
                    tests_by_testcase[testcase_name] = []
                tests_by_testcase[testcase_name].append(test_info)

        return tests_by_testcase

    def _report_testsuite(suite_name, tests, xml_document):
        "Appends the testsuite section to the XML document."
        testsuite = xml_document.createElement('testsuite')
        xml_document.appendChild(testsuite)

        testsuite.setAttribute('name', suite_name)
        testsuite.setAttribute('tests', str(len(tests)))

        testsuite.setAttribute('time', '%.3f' % \
            sum(map(lambda e: e.get_elapsed_time(), tests)))

        failures = filter(lambda e: e.outcome == _TestInfo.FAILURE, tests)
        testsuite.setAttribute('failures', str(len(failures)))

        errors = filter(lambda e: e.outcome == _TestInfo.ERROR, tests)
        testsuite.setAttribute('errors', str(len(errors)))

        return testsuite

    _report_testsuite = staticmethod(_report_testsuite)

    def _test_method_name(test_method):
        "Returns the test method name."
        test_id = test_method.id()
        return test_id.split('.')[-1]

    _test_method_name = staticmethod(_test_method_name)

    def _report_testcase(suite_name, test_result, xml_testsuite, xml_document):
        "Appends a testcase section to the XML document."
        testcase = xml_document.createElement('testcase')
        xml_testsuite.appendChild(testcase)

        testcase.setAttribute('classname', suite_name)
        testcase.setAttribute('name', _XMLTestResult._test_method_name(test_result.test_method))
        testcase.setAttribute('time', '%.3f' % test_result.get_elapsed_time())

        if (test_result.outcome != _TestInfo.SUCCESS):
            elem_name = ('failure', 'error')[test_result.outcome - 1]
            failure = xml_document.createElement(elem_name)
            testcase.appendChild(failure)

            failure.setAttribute('type', test_result.err[0].__name__)
            failure.setAttribute('message', str(test_result.err[1]))

            error_info = test_result.get_error_info()
            _XMLTestResult._add_cdata(xml_document, error_info, failure)

    _report_testcase = staticmethod(_report_testcase)

    def _report_output(test_runner, xml_testsuite, xml_document):
        "Appends the system-out and system-err sections to the XML document."
        systemout = xml_document.createElement('system-out')
        xml_testsuite.appendChild(systemout)

        stdout = test_runner.stdout.getvalue()
        _XMLTestResult._add_cdata(xml_document, stdout, systemout)

        systemerr = xml_document.createElement('system-err')
        xml_testsuite.appendChild(systemerr)

        stderr = test_runner.stderr.getvalue()
        _XMLTestResult._add_cdata(xml_document, stderr, systemerr)

    _report_output = staticmethod(_report_output)

    def _add_cdata(xml_document, content, append_to=None):
        """
        Protect from CDATA inside CDATA
        """
        results = list()
        content = content.split(']]>')
        for index, entry in enumerate(content):
            if not entry:
                continue
            if index < len(content) - 1:
                entry += ']]'
            results.append(xml_document.createCDATASection(entry))
            if index < len(content) - 1:
                results.append(xml_document.createCDATASection('>'))
        if append_to:
            for result in results:
                append_to.appendChild(result)

    _add_cdata = staticmethod(_add_cdata)

    def generate_reports(self, test_runner):
        "Generates the XML reports to a given XMLTestRunner object."
        all_results = self._get_info_by_testcase()

        if not os.path.exists(test_runner.output_dir):
            os.makedirs(test_runner.output_dir)

        for suite, tests in all_results.items():
            doc = Document()

            # Build the XML file
            testsuite = _XMLTestResult._report_testsuite(suite, tests, doc)
            for test in tests:
                _XMLTestResult._report_testcase(suite, test, testsuite, doc)
            _XMLTestResult._report_output(test_runner, testsuite, doc)
            xml_content = doc.toprettyxml(indent='\t')

            report_file = file('%s%sTEST-%s.xml' % (test_runner.output_dir, os.sep, suite), 'w')
            try:
                report_file.write(xml_content)
            finally:
                report_file.close()


class XMLTestRunner(DjangoTestRunner):
    """
    A test result class that can express test results in a XML report.
    """
    def __init__(self, output_dir, debug=False, with_reports=True, **kwargs):
        super(XMLTestRunner, self).__init__(**kwargs)
        self.with_reports = with_reports
        self.debug = debug
        self.output_dir = output_dir

    def _makeResult(self):
        return _XMLTestResult(self.stream, self.descriptions, self.verbosity)

    def _patch_standard_output(self):
        """Replace the stdout and stderr streams with string-based streams
        in order to capture the tests' output.
        """
        (self.old_stdout, self.old_stderr) = (sys.stdout, sys.stderr)
        (self.stdout, self.stderr) = (StringIO(), StringIO())

        if not self.debug:
            (sys.stdout, sys.stderr) = (self.stdout, self.stderr)

    def _restore_standard_output(self):
        "Restore the stdout and stderr streams."
        (sys.stdout, sys.stderr) = (self.old_stdout, self.old_stderr)

    def run(self, test):
        try:
            self._patch_standard_output()
            result = super(XMLTestRunner, self).run(test)
            if self.with_reports:
                result.generate_reports(self)
        finally:
            self._restore_standard_output()
        return result


class CITestSuiteRunner(DjangoTestSuiteRunner):
    """
    Continues integration test runner
    """
    def __init__(self, output_dir, debug=False, with_reports=True, **kwargs):
        super(CITestSuiteRunner, self).__init__(**kwargs)
        self.with_reports = with_reports
        self.debug = debug
        self.output_dir = output_dir

    def setup_test_environment(self, **kwargs):
        super(CITestSuiteRunner, self).setup_test_environment()
        signals.setup_test_environment.send(sender=self)

    def teardown_test_environment(self, **kwargs):
        super(CITestSuiteRunner, self).teardown_test_environment()
        signals.teardown_test_environment.send(sender=self)

    def setup_databases(self):
        if 'south' in settings.INSTALLED_APPS:
            from south.management.commands import patch_for_test_db_setup  # pylint: disable=F0401
            patch_for_test_db_setup()
        return super(CITestSuiteRunner, self).setup_databases()

    def build_suite(self, test_labels, **kwargs):
        suite = unittest.TestSuite()
        signals.build_suite.send(sender=self, suite=suite)
        return suite

    def run_tests(self, test_labels, extra_tests=None, **kwargs):
        self.setup_test_environment()
        suite = self.build_suite(test_labels, extra_tests=extra_tests)
        if suite.countTestCases():
            old_config = self.setup_databases()
            result = self.run_suite(suite)
            self.teardown_databases(old_config)
            self.teardown_test_environment()
            return self.suite_result(suite, result)
        else:
            self.teardown_test_environment()
            return True

    def run_suite(self, suite, **kwargs):
        signals.before_suite_run.send(sender=self)
        result = XMLTestRunner(
            verbosity=self.verbosity,
            output_dir=self.output_dir,
            debug=self.debug,
            with_reports=self.with_reports).run(suite)
        signals.after_suite_run.send(sender=self)

        return result



# -*- coding: utf-8 -*-
from django_jenkins.tasks.windmill_tests import WindmillTestCase


class TestWMClickPage(WindmillTestCase):
    def test_wmclick(self):
        self.windmill.open('wm_test_click')
        self.windmill.waits.forPageLoad(timeout=u'5000')
        self.windmill.click(id='wm_click')
        self.windmill.asserts.assertText(validator=u'Button clicked', id=u'wm_target')



# -*- coding: utf-8 -*-
import re
import os
import sys
import pep8
from optparse import make_option
from django_jenkins.functions import relpath
from django_jenkins.tasks import BaseTask, get_apps_locations


class Task(BaseTask):
    option_list = [make_option("--pep8-exclude",
                               dest="pep8-exclude", default=pep8.DEFAULT_EXCLUDE,
                               help="exclude files or directories which match these "
                               "comma separated patterns (default: %s)" %
                               pep8.DEFAULT_EXCLUDE),
                   make_option("--pep8-select", dest="pep8-select",
                               help="select errors and warnings (e.g. E,W6)"),
                   make_option("--pep8-ignore", dest="pep8-ignore",
                               help="skip errors and warnings (e.g. E4,W)")]

    def __init__(self, test_labels, options):
        super(Task, self).__init__(test_labels, options)
        self.test_all = options['test_all']

        if options.get('pep8_file_output', True):
            output_dir = options['output_dir']
            if not os.path.exists(output_dir):
                os.makedirs(output_dir)
            self.output = open(os.path.join(output_dir, 'pep8.report'), 'w')
        else:
            self.output = sys.stdout

        self.pep8_options = ['--exclude=%s' % options['pep8-exclude']]
        if options['pep8-select']:
            self.pep8_options.append('--select=%s' % options['pep8-select'])
        if options['pep8-ignore']:
            self.pep8_options.append('--ignore=%s' % options['pep8-ignore'])

    def teardown_test_environment(self, **kwargs):
        locations = get_apps_locations(self.test_labels, self.test_all)
        pep8.process_options(self.pep8_options + locations)

        # run pep8 tool with captured output
        def report_error(instance, line_number, offset, text, check):
            code = text[:4]
            if pep8.ignore_code(code):
                return
            message = re.sub(r'([WE]\d+)', r'[\1] PEP8:', text)
            sourceline = instance.line_offset + line_number
            self.output.write('%s:%s: %s\n' % (instance.filename, sourceline, message))
        pep8.Checker.report_error = report_error

        for location in locations:
            pep8.input_dir(relpath(location), runner=pep8.input_file)

        self.output.close()



# -*- coding: utf-8 -*-
import os.path
import subprocess

def relpath(path, start=os.path.curdir):
    """
    Return a relative version of a path

    Backport from Python2.7
    """
    if not path:
        raise ValueError("no path specified")
    
    start_list = os.path.abspath(start).split(os.path.sep)
    path_list = os.path.abspath(path).split(os.path.sep)
    
    # Work out how much of the filepath is shared by start and path.
    i = len(os.path.commonprefix([start_list, path_list]))
    
    rel_list = [os.path.pardir] * (len(start_list) - i) + path_list[i:]
    if not rel_list:
        return os.path.curdir
    return os.path.join(*rel_list)


def check_output(*popenargs, **kwargs):
    """
    Backport from Python2.7 
    """
    if 'stdout' in kwargs:
        raise ValueError('stdout argument not allowed, it will be overridden.')
    process = subprocess.Popen(stdout=subprocess.PIPE, *popenargs, **kwargs)
    output, unused_err = process.communicate()
    retcode = process.poll()
    if retcode:
        cmd = kwargs.get("args")
        if cmd is None:
            cmd = popenargs[0]

        exception = subprocess.CalledProcessError(retcode, cmd)
        exception.output = output
        raise exception
    return output



# -*- coding: utf-8 -*-
from django_jenkins.management.commands import TaskListCommand


class Command(TaskListCommand):
    help = "Run windmill unittests"

    def get_task_list(self):
        return ('django_jenkins.tasks.windmill_tests',)



import os, sys

PROJECT_ROOT = os.path.abspath(os.path.dirname(__file__))

DEBUG=True
TEMPLATE_DEBUG=DEBUG
ROOT_URLCONF = 'test_app.urls'

TEMPLATE_LOADERS = (
    'django.template.loaders.app_directories.load_template_source',
)

PROJECT_APPS = (
    'django.contrib.sessions', # just to ensure that dotted apps test works
    'django_jenkins',
    'test_app',
)

INSTALLED_APPS = (
    'django.contrib.contenttypes',
) + PROJECT_APPS


DATABASE_ENGINE = 'sqlite3'
DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.%s' % DATABASE_ENGINE,
        }
}

JENKINS_TASKS = (
    'django_jenkins.tasks.with_coverage',
    'django_jenkins.tasks.django_tests',
    'django_jenkins.tasks.run_pep8',
    'django_jenkins.tasks.run_pyflakes',
    'django_jenkins.tasks.run_jslint',
)

JSLINT_CHECKED_FILES = [os.path.join(PROJECT_ROOT, 'static/js/test.js')]

# python > 2.4
if sys.version_info[1] > 4:
    JENKINS_TASKS += ('django_jenkins.tasks.run_pylint',
                      'django_jenkins.tasks.windmill_tests')



from django.core import mail
from django.test import TestCase


class SaintyChecks(TestCase):
    def test_mailbox_stubs_not_broken(self):
        mail.send_mail('Test subject', 'Test message', 'nobody@kenkins.com',
                       ['somewhere@nowhere.com'])
        self.assertTrue(1, len(mail.outbox))




# -*- coding: utf-8 -*-
import os
import sys
from optparse import make_option
from django.conf import settings
from django_jenkins.functions import check_output, relpath
from django_jenkins.tasks import BaseTask, get_apps_locations

class Task(BaseTask):
    option_list = [make_option("--jslint-interpreter",
                               dest="jslint_interpreter",
                               help="Javascript interpreter for running jslint"),
                   make_option("--jslint-implementation",
                               dest="jslint_implementation",
                               help="Full path to fulljslint.js, by default used build-in"),
                   make_option("--jslint-exclude",
                               dest="jslint_exclude", default="",
                               help="Exclude patterns")]

    def __init__(self, test_labels, options):
        super(Task, self).__init__(test_labels, options)
        self.test_all = options['test_all']

        root_dir = os.path.normpath(os.path.dirname(__file__))

        self.intepreter = options['jslint_interpreter'] or \
                          getattr(settings, 'JSLINT_INTERPRETER', 'rhino')

        self.implementation = options['jslint_implementation']
        if not self.implementation:
            self.implementation = os.path.join(root_dir, 'jslint', 'fulljslint.js')

        if options.get('jslint_file_output', True):
            output_dir = options['output_dir']
            if not os.path.exists(output_dir):
                os.makedirs(output_dir)
            self.output = open(os.path.join(output_dir, 'jslint.report'), 'w')
        else:
            self.output = sys.stdout

        self.runner = os.path.join(root_dir, 'jslint_runner.js')
        self.exclude = options['jslint_exclude']

    def teardown_test_environment(self, **kwargs):    
        for path in self.static_files_iterator():
            jslint_output = check_output(
                [self.intepreter, self.runner, self.implementation, relpath(path)])
            self.output.write(jslint_output)

    def static_files_iterator(self):
        locations = get_apps_locations(self.test_labels, self.test_all)

        def in_tested_locations(path):
            for location in locations:
                if path.startswith(location):
                    return True
            return False
        
        if hasattr(settings, 'JSLINT_CHECKED_FILES'):
            for path in settings.JSLINT_CHECKED_FILES:
                yield path
                    
        if 'django.contrib.staticfiles' in settings.INSTALLED_APPS:
            # use django.contrib.staticfiles
            from django.contrib.staticfiles import finders

            for finder in finders.get_finders():
                for path, _ in finder.list(self.exclude):
                    if path.endswith('.js') and in_tested_locations(path):
                        yield path
        else:
            # scan apps directories for static folders
            for location in locations:
                for dirpath, dirnames, filenames in os.walk(os.path.join(location, 'static')):
                    for filename in filenames:
                        if filename.endswith('.js') and in_tested_locations(os.path.join(dirpath, filename)):
                            yield os.path.join(dirpath, filename)

